<?php
/*
Plugin Name: En Garde
Plugin URI: http://blog.tremlas.com/en-garde-plugin-details/
Description: Adds En Garde shortcodes
Version: 3.26.0
Author: David Vines
Author URI: http://tremlas.com
License:  Creative Commons License
*/
/* © Copyright 2014, 2017 David Vines

   Creative Commons License

   This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
*/

// Debugging function in case we need it
if ( !function_exists('djv_write_log') ) {
	function djv_write_log ( $log )  {
		if ( true === WP_DEBUG ) {
			if ( is_array( $log ) || is_object( $log ) ) {
				error_log( print_r( $log, true ) );
			} else {
				error_log( $log );
			}
		}
	}
}

function djv_engarde_install_rowname_table() {
    global $wpdb;
    $djv_engarde_rowname_table_version = get_option( 'djv_engarde_rowname_table_version', '' ); // Table version NOT plugin version
    
    if ( $djv_engarde_rowname_table_version != "1.0" ) {
        $table_name = $wpdb->prefix . 'engarde_rownames';
    
        $charset_collate = $wpdb->get_charset_collate();
    
        $sql = "CREATE TABLE $table_name (
                id int(11) NOT NULL,
                name varchar(20) NOT NULL,
                PRIMARY KEY  (id)
                )  $charset_collate;";
        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
        
        $wpdb->replace($table_name, array( 'id' =>   10, 'name' => 'Support' ) );
        $wpdb->replace($table_name, array( 'id' =>   20, 'name' => 'Regiment' ) );
        $wpdb->replace($table_name, array( 'id' =>   30, 'name' => 'Rank' ) );
        $wpdb->replace($table_name, array( 'id' =>   40, 'name' => 'Club' ) );
        $wpdb->replace($table_name, array( 'id' =>   50, 'name' => 'Location' ) );
        $wpdb->replace($table_name, array( 'id' =>  100, 'name' => 'Week 1' ) );
        $wpdb->replace($table_name, array( 'id' =>  200, 'name' => 'Week 2' ) );
        $wpdb->replace($table_name, array( 'id' =>  300, 'name' => 'Week 3' ) );
        $wpdb->replace($table_name, array( 'id' =>  400, 'name' => 'Week 4' ) );
        $wpdb->replace($table_name, array( 'id' => 1000, 'name' => 'Other Notes' ) );
        
        update_option( 'djv_engarde_rowname_table_version', '1.0' );
    }
}
register_activation_hook( __FILE__, 'djv_engarde_install_rowname_table');

function djv_engarde_install_orderrows_table() {
    global $wpdb;
    $djv_engarde_orderrows_table_version = get_option( 'djv_engarde_orderrows_table_version', '' ); // Table version NOT plugin version
 
    if ( $djv_engarde_orderrows_table_version != "1.0" ) {
        $table_name = $wpdb->prefix . 'engarde_orderrows';
 
        $charset_collate = $wpdb->get_charset_collate();
        
        $sql = "CREATE TABLE $table_name (
                id bigint(20) NOT NULL AUTO_INCREMENT,
                page_id bigint(20) NOT NULL,
                month varchar(20) NOT NULL,
                rowid int(11) NOT NULL,
                action varchar(255) NOT NULL,
                notes longtext NOT NULL,
                effect longtext NOT NULL,
                version varchar(20) NOT NULL,
                PRIMARY KEY  (id),
                KEY row (rowid),
                KEY charmonth (page_id, month) USING BTREE,
                UNIQUE KEY recordkey (page_id, month, rowid) USING BTREE
        )  $charset_collate;";
        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
        
        update_option( 'djv_engarde_orderrows_table_version', '1.0' );
    }
}
register_activation_hook( __FILE__, 'djv_engarde_install_orderrows_table');

// Initial month
add_option( 'djv_engarde_options', array(
		'month' => 'février 1924',
        'femmes' => 'Auriane Delaunay,Flora Pernet',
        'email' => 'engarde@blog.tremlas.com',
        'closed' => false,
        'extra' => array(
          array( 'text' => '', 'notes' => ''), array( 'text' => '', 'notes' => ''), array( 'text' => '', 'notes' => ''), array( 'text' => '', 'notes' => '')
        )
	)
);


// An admin page to allow the month to be modified
class DjvengardeSettingsPage
{
	private $options;
	
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
		add_action( 'admin_init', array( $this, 'page_init' ) );
	}
	
	public function add_plugin_page() {
		add_options_page(
			'En Garde Admin',
			'En Garde',
			'manage_options',
			'djv-engarde-setting-admin',
			array( $this, 'create_admin_page' ) 
		);
	}
	
	public function create_admin_page() {
		$this->options = get_option('djv_engarde_options');
		?>
		<div class="wrap">
			<?php screen_icon(); ?>
			<h2>En Garde Options</h2>
			<form method="post" action="options.php">
			<?php
				settings_fields('djv_engarde_option_group');
				do_settings_sections('djv-engarde-setting-admin');
				submit_button(); 
			?>
			</form>
		</div>
		<?php
	}
	
	public function page_init() {
		register_setting(
			'djv_engarde_option_group',
			'djv_engarde_options',
			array( $this, 'sanitize' )
		);
		add_settings_section(
			'djv_engarde_settings_section_id',
			'En Garde Settings',
			array( $this, 'print_section_info' ),
			'djv-engarde-setting-admin'
		);
		add_settings_field ( 'djv_engarde_settings_open',
		    'Are En Garde Orders being accepted?',
		    array ( $this, 'open_callback' ),
		    'djv-engarde-setting-admin',
		    'djv_engarde_settings_section_id' 
		);
		add_settings_field(
			'djv_engarde_settings_month',
			'The month for which orders are expected',
			array( $this, 'month_callback' ),
			'djv-engarde-setting-admin',
			'djv_engarde_settings_section_id'
		);
		add_settings_field(
			'djv_engarde_settings_femmes',
			'Las femmes fatales',
			array( $this, 'femmes_callback' ),
			'djv-engarde-setting-admin',
			'djv_engarde_settings_section_id'
		);
		add_settings_field(
			'djv_engarde_settings_email',
			'Email to which to submit orders',
			array( $this, 'email_callback' ),
			'djv-engarde-setting-admin',
			'djv_engarde_settings_section_id'
		);
		for( $i = 1; $i < 5; $i++) {
		 add_settings_field(
		   'djv_engarde_settings_week_' . $i . '_activity',
		   'Extra activity for week ' . $i . ' (Set to empty or blank to omit this from the list of choices)',
		   array( $this, 'week_' . $i . '_activity_callback' ),
		   'djv-engarde-setting-admin',
		   'djv_engarde_settings_section_id'
		   );
		 add_settings_field(
		   'djv_engarde_settings_week_' . $i . '_notes',
		   'Extra activity notes for week ' . $i . ' (The text shown in the description column if the activity is selected)',
		   array( $this, 'week_' . $i . '_notes_callback' ),
		   'djv-engarde-setting-admin',
		   'djv_engarde_settings_section_id'
		   );
		 }
	}
	
	public function sanitize( $input ) {
		$new_input = array();
		$open = (isset ( $input ['open'] ) && true == $input ['open'] ? true : false);
		$new_input['closed'] = !($open); // Note opposite sense
		if ( isset( $input['month'] ) ) {
			$new_input['month'] = $input['month'];
		}
		if ( isset( $input['femmes'] ) ) {
			$new_input['femmes'] = $input['femmes'];
		}
		if ( isset( $input['email'] ) && (filter_var($input['email'], FILTER_VALIDATE_EMAIL) ) ) {
			$new_input['email'] = $input['email'];
		}
		$new_input['extra'] = array(
		  array( 'text' => '', 'notes' => ''), array( 'text' => '', 'notes' => ''), array( 'text' => '', 'notes' => ''), array( 'text' => '', 'notes' => '')
		);
		for( $i = 1; $i < 5; $i++) {
		 if ( isset( $input['week_' . $i . '_activity'] ) ) {
		  $new_input['extra'][$i-1]['text'] = $input['week_' . $i . '_activity'];
		 }
		 if ( isset( $input['week_' . $i . '_notes'] ) ) {
		  $new_input['extra'][$i-1]['notes'] = $input['week_' . $i . '_notes'];
		 }
		}
		return $new_input;
	}
	
	public function print_section_info() {
		print "Enter the settings for the En Garde plugin.";
	}
	
	public function open_callback() {
	    $checked = 'checked="checked"';
	    if ($this->options ['closed']) {
	        $checked = '';
	    }
	    $html = '<input type="checkbox" id="djv_engarde_options_open" name="djv_engarde_options[open]" value="1"' . $checked . '>' . PHP_EOL;
	    $html .= '<label for="djv_engarde_options_open">Accepting orders for En Garde?</label>' . PHP_EOL;

	    echo $html;
	}
    public function month_callback() {
        $plugin_url = plugin_dir_url ( __FILE__ );
        wp_enqueue_script ( 'djv_engarde_dates', $plugin_url . 'js/engarde/dates.js' );
        wp_enqueue_script ( 'djv_engarde_editdate', $plugin_url . 'js/engarde/editdate.js', array ( 'djv_engarde_dates' ) );

        $html = '<button type="submit" id="gamemonthprevious">&lt;</button>' . PHP_EOL;
        $html .= '<span id="gamemonth" style="display: inline-block; width 32ch;"></span>' . PHP_EOL;
        $html .= '<button type="submit" id="gamemonthnext">&gt;</button>' . PHP_EOL;
        $html .= '<div style="display: none"><textarea id="newgamemonth" name="djv_engarde_options[month]">' . $this->options ['month'] . '</textarea></div>' . PHP_EOL;

        echo $html;
    }
	
    public function email_callback() {
     $html = '<textarea name="djv_engarde_options[email]" rows="1" cols="60">';
     $html .= isset( $this->options['email'] ) ? $this->options['email'] : '';
     $html .= '</textarea>';
     echo $html;
	}
	
	public function femmes_callback() {
		$settings = array(
			'textarea_name' => 'djv_engarde_options[femmes]',
			'teeny' => true,
			'textarea_rows' => 5,
			'tabindex' => 1,
			'media_buttons' => false
		);
		wp_editor(
			isset( $this->options['femmes'] ) ? $this->options['femmes'] : '',
			'femmes',
			$settings
		);
	}
	
	public function week_1_activity_callback () { $this->week_n_activity_callback(1); }
	public function week_2_activity_callback () { $this->week_n_activity_callback(2); }
	public function week_3_activity_callback () { $this->week_n_activity_callback(3); }
	public function week_4_activity_callback () { $this->week_n_activity_callback(4); }
	public function week_n_activity_callback( $n ) {
     $html = '<textarea name="djv_engarde_options[week_' . $n . '_activity]" rows="1" cols="80">';
     $html .= isset( $this->options['extra'] ) ? $this->options['extra'][$n-1]['text'] : '';
     $html .= '</textarea>';
     echo $html;
	}
	
	public function week_1_notes_callback () { $this->week_n_notes_callback(1); }
	public function week_2_notes_callback () { $this->week_n_notes_callback(2); }
	public function week_3_notes_callback () { $this->week_n_notes_callback(3); }
	public function week_4_notes_callback () { $this->week_n_notes_callback(4); }
	public function week_n_notes_callback( $n ) {
	 $settings = array(
	   'textarea_name' => 'djv_engarde_options[week_' . $n . '_notes]',
	   'teeny' => true,
	   'textarea_rows' => 5,
	   'tabindex' => 1,
	   'media_buttons' => false
	 );
	 wp_editor(
	   isset( $this->options['extra'] ) ? $this->options['extra'][$n-1]['notes'] : '',
	   'week_' . $n . '_notes',
	   $settings
	   );
	}
}
if (is_admin()) {
	$djv_engarde_settings_page = new DjvengardeSettingsPage();
}

// Also provide a link on the plugins listings page
function djv_add_engarde_plugin_action_links( $links ) {
	return array_merge(
			array( 'settings' => '<a href="' . get_bloginfo( 'wpurl' ) . '/wp-admin/options-general.php?page=djv-engarde-setting-admin">Settings</a>'
			),
			$links
		);
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'djv_add_engarde_plugin_action_links' );

/** 
 * Shortcode to add the version of the plugin
 */
function djv_engarde_engardeVersion_shortcode() {
	return 'En Garde Version 3.26.0';
} 
add_shortcode( 'enGardeVersion', 'djv_engarde_engardeVersion_shortcode' );

function djv_engarde_engardeCharacter_attr_shortcode_general( $name, $attr ) {
 if ( isset($attr) && isset( $attr['is'] ) ) {
  $cur = $attr['is'];
 } else if ( isset($attr) && isset( $attr['has'] ) ) {
  $cur = $attr['has'];
 } else {
  $cur = 'unknown';
 }
 $notes = '';
 if ( isset($attr) && isset( $attr['initial'] ) ) {
  $notes .= 'Started at ' . $attr['initial'];
 }
 if ( isset($attr) && isset( $attr['practice'] ) ) {
  if ( $notes !== '' ) { $notes .= '. '; }
  $notes .= $attr['practice'] . ' of practice';
 } else if ( isset($attr) && isset( $attr['notes'] ) ) {
  if ( $notes !== '' ) { $notes .= '. '; }
  $notes .= $attr['notes'];
 }
 return '<tr><td>' . $name . '</td><td>' . $cur . '</td><td>'  . $notes . '</td></tr>';
}

function djv_engarde_engardeCharacter_weapon_shortcode_general( $name, $attr ) {
 $skill = get_post_meta( get_the_ID(), $name, true);
 if ( $skill == "" ) {
  if ( isset( $attr['practice'] ) ) {
   $skill = $attr['is'] . '.' . explode(' ',$attr['practice'])[0];
  } else {
   $skill = $attr['is'] . '.0';
  }
  add_post_meta( get_the_ID(), $name, $skill, true);
 }
 $values = explode('.',$skill);
 $attr['is'] = $values[0];
 if ( $values[1] == '0' ) {
  unset($attr['practice']);
 } else if ($values[1] == '1') {
  $attr['practice'] = $values[1] . ' week';
 } else {
  $attr['practice'] = $values[1] . ' weeks';
 }
 return djv_engarde_engardeCharacter_attr_shortcode_general($name . ' Expertise', $attr);
}
 
function djv_engarde_engardeCharacter_str_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Strength', $attr );
}

function djv_engarde_engardeCharacter_rapier_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_weapon_shortcode_general( 'Rapier', $attr );
}

function djv_engarde_engardeCharacter_sabre_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_weapon_shortcode_general( 'Sabre', $attr );
}

function djv_engarde_engardeCharacter_cutlass_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_weapon_shortcode_general( 'Cutlass', $attr );
}

function djv_engarde_engardeCharacter_con_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Constitution', $attr );
}

function djv_engarde_engardeCharacter_end_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Endurance', $attr );
}

function djv_engarde_engardeCharacter_ma_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Military Ability', $attr );
}

function djv_engarde_engardeCharacter_cash_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Cash', $attr );
}

function djv_engarde_engardeCharacter_allowance_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Allowance and/or Pension', $attr );
}

function djv_engarde_engardeCharacter_club_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Club', $attr );
}

function djv_engarde_engardeCharacter_regiment_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Regiment &amp; Rank', $attr );
}

function djv_engarde_engardeCharacter_appointment_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Appointment', $attr );
}

function djv_engarde_engardeCharacter_mistress_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Female Companion', $attr );
}

function djv_engarde_engardeCharacter_mentions_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Mentions in Dispatches', $attr );
}

function djv_engarde_engardeCharacter_stable_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Stable', $attr );
}

function djv_engarde_engardeCharacter_investment_shortcode( $attr ) {
if ( isset($attr) && isset( $attr['notes'] ) ) {
  $notes = $attr['notes'];
 } else {
  $notes = '';
 }
 if ( isset($attr) && isset( $attr['arms'] ) ) {
  return '<tr><td>Investments in Arms Companies</td><td>' . $attr['arms'] . ' Crowns</td><td>'  . $notes . '</td></tr>';
 } else if ( isset($attr) && isset( $attr['finance'] ) ) {
  return '<tr><td>Investments in Finance Houses</td><td>' . $attr['finance'] . ' Crowns</td><td>'  . $notes . '</td></tr>';
 } else if ( isset($attr) && isset( $attr['commerce'] ) ) {
  return '<tr><td>Investments in Commercial Businesses</td><td>' . $attr['commerce'] . ' Crowns</td><td>'  . $notes . '</td></tr>';
 } else if ( isset($attr) && isset( $attr['property'] ) ) {
  return '<tr><td>Investments in Parisian Property</td><td>' . $attr['property'] . ' Crowns</td><td>'  . $notes . '</td></tr>';
 } else if ( isset($attr) && isset( $attr['tulips'] ) ) {
  return '<tr><td>Investments in Tulip Farms</td><td>' . $attr['tulips'] . ' Crowns</td><td>'  . $notes . '</td></tr>';
 }
}

function djv_engarde_engardeCharacter_level_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Social Level', $attr );
}

function djv_engarde_engardeCharacter_location_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Location', $attr );
}

function djv_engarde_engardeCharacter_player_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Player', $attr );
}

function djv_engarde_engardeCharacter_favours_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Favours', $attr );
}

function djv_engarde_engardeCharacter_battleOutcome_shortcode( $attr ) {
 if ( isset($attr) && isset( $attr['is'] ) ) {
  $cur = $attr['is'];
 } else {
  $cur = 'unknown';
 }
 return '<tr style="display: none;"><td>Battle Outcome</td><td>' . $cur . '</td><td></td></tr>';
}

function djv_engarde_engardeCharacter_shortcode( $attr, $content = null ) {
 if ( isset($attr) && isset( $attr['id'] ) ) {
  $id = $attr['id'];
 } else {
  $id = '';
 }
 
 $text = '' . PHP_EOL;
 
 $text .= '<table id="djv-engarde-stats-' . $id . '">' . PHP_EOL;
 $text .= '<tr><th>Attribute</th><th>Currently</th><th>Notes</th></tr>' . PHP_EOL;
 
 $shorts = array( 'str', 'rapier', 'sabre', 'cutlass', 'con', 'end', 'ma', 'cash', 'allowance', 'club', 'regiment', 'appointment', 'mistress', 'mentions', 'stable', 'level', 'location', 'player', 'favours', 'battleOutcome', 'investment' );
 
 foreach ($shorts as $code) {
  add_shortcode( $code, 'djv_engarde_engardeCharacter_' . $code . '_shortcode' );
 }
 
 $text .= do_shortcode( $content );
 $text = preg_replace( '!<br ?/>!', '', $text );
 
 foreach ($shorts as $code) {
  remove_shortcode( $code );
 }
 
 $text .= '</table>' . PHP_EOL;
  
 return $text;
}
add_shortcode( 'enGardeCharacter', 'djv_engarde_engardeCharacter_shortcode' );

/**
 * Shortcode to add order template
 */
function djv_engarde_engardeOrderTemplate_shortcode( $attr, $content = null ) {
 $plugin_url = plugin_dir_url( __FILE__ );
 wp_enqueue_script( 'djv_engarde_utils', $plugin_url.'js/engarde/utils.js' );
 wp_enqueue_script( 'djv_engarde_polyfills', $plugin_url.'js/engarde/polyfills.js' );
 wp_enqueue_script( 'djv_engarde_row', $plugin_url.'js/engarde/row.js', array( 'djv_engarde_polyfills' ) );
 wp_enqueue_script( 'djv_engarde_regiment', $plugin_url.'js/engarde/regiment.js' );
 wp_enqueue_script( 'djv_engarde_club', $plugin_url.'js/engarde/club.js', array( 'djv_engarde_regiment' ) );
 wp_enqueue_script( 'djv_engarde_support_row', $plugin_url.'js/engarde/support_row.js', array( 'djv_engarde_row', 'djv_engarde_utils' ) );
 wp_enqueue_script( 'djv_engarde_regiment_row', $plugin_url.'js/engarde/regiment_row.js', array( 'djv_engarde_row', 'djv_engarde_regiment' ) );
 wp_enqueue_script( 'djv_engarde_rank_row', $plugin_url.'js/engarde/rank_row.js', array( 'djv_engarde_row' ) );
 wp_enqueue_script( 'djv_engarde_club_row', $plugin_url.'js/engarde/club_row.js', array( 'djv_engarde_row', 'djv_engarde_club' ) );
 wp_enqueue_script( 'djv_engarde_location_row', $plugin_url.'js/engarde/location_row.js', array( 'djv_engarde_row' ) );
 wp_enqueue_script( 'djv_engarde_other_row', $plugin_url.'js/engarde/other_row.js', array( 'djv_engarde_row' ) );
 wp_enqueue_script( 'djv_engarde_week_row', $plugin_url.'js/engarde/week_row.js', array( 'djv_engarde_row', 'djv_engarde_club', 'djv_engarde_regiment', 'djv_engarde_utils' ) );
 wp_enqueue_script( 'djv_engarde_engarde', $plugin_url.'js/engarde/engarde.js', array( 'djv_engarde_polyfills', 'djv_engarde_support_row', 'djv_engarde_regiment_row', 'djv_engarde_rank_row', 'djv_engarde_club_row', 'djv_engarde_location_row', 'djv_engarde_other_row', 'djv_engarde_week_row' ) );
 wp_enqueue_style( 'djv_engarde_enGarde_style', $plugin_url.'css/engarde.css' );
 
 if ( isset($attr) && isset( $attr['id'] ) ) {
  $id = $attr['id'];
 } else {
  $id = 'unknown';
 }
 
 $lastordersfor = get_post_meta( get_the_ID(), 'lastordersfor', true);
 $lastorderson = get_post_meta( get_the_ID(), 'lastorderson', true);
 if ( ( $lastordersfor == "" ) || ( $lastorderson == "") ) {
  $lastorders = "";
 } else {
  $lastorders = "<p>Orders for " . $lastordersfor . " were last submitted " . $lastorderson . '.</p>';
 }
 
 $keys = array();
 $values = array();
 
 $keys[0]  = '@@@ID@@@';            $values[0]  = 'djv-engarde-' . $id;
 $keys[1]  = '@@@WK@@@';            $values[1]  = '0';
 $keys[2]  = '@@@FEMMES@@@';        $values[2]  = '';
 $keys[3]  = '@@@ADMINURL@@@';      $values[3]  = esc_url( admin_url('admin-post.php'));
 $keys[4]  = '@@@TITLE@@@';         $values[4]  = esc_html( get_the_title() );
 $keys[5]  = '@@@NONCE@@@';         $values[5]  = wp_nonce_field('submit_orders', 'nonce', true, false);
 $keys[6]  = '@@@POSTID@@@';        $values[6]  = get_the_ID();
 $keys[7]  = '@@@MONTH@@@';         $values[7]  = get_option('djv_engarde_options')['month'];
 $keys[8]  = '@@@ACTIVITYTEXT@@@';  $values[8]  = '';
 $keys[9]  = '@@@ACTIVITYNOTES@@@'; $values[9]  = '';
 $keys[10] = '@@@LASTORDERS@@@';    $values[10] = $lastorders;
 
 $dir = plugin_dir_path(__FILE__);
 $pre      = file_get_contents($dir . '/html/pre.fragment');
 $week     = file_get_contents($dir . '/html/week.fragment');
 $post     = file_get_contents($dir . '/html/post.fragment');
 $preform  = file_get_contents($dir . '/html/preform.fragment');
 $weekform = file_get_contents($dir . '/html/weekform.fragment');
 $postform = file_get_contents($dir . '/html/postform.fragment');
 
 $femmes = get_option('djv_engarde_options')['femmes'];
 $femmes = str_replace(array("\r", "\n"), '', $femmes); // To remove any line breaks introduced for readability on the admin panel :)
 $femmes = explode(';', $femmes);
 sort($femmes);
 $values[2] = '<option>' . implode('</option>' . PHP_EOL . '<option>', $femmes) . '</option>' . PHP_EOL;
 
 $text = '';
 $text .= str_replace($keys, $values, $pre);
 $extra = get_option('djv_engarde_options')['extra'];
 for( $i = 1; $i < 5; $i++) {
  $values[1] = strval($i);
  if ( isset( $extra ) ) {
   $values[8] = $extra[$i-1]['text'];
   $values[9] = $extra[$i-1]['notes'];
  }
  $text .= str_replace($keys, $values, $week);
 }
 $text .= str_replace($keys, $values, $post);
 $text .= str_replace($keys, $values, $preform);
 for( $i = 1; $i < 5; $i++) {
  $values[1] = strval($i);
  $text .= str_replace($keys, $values, $weekform);
 }
 $text .= str_replace($keys, $values, $postform);
 
 return $text;
}
add_shortcode( 'enGardeOrderTemplate', 'djv_engarde_engardeOrderTemplate_shortcode' );

function djv_engarde_verify_passcode( $postid, $name, $passcode ) {
 if ( empty( $postid ) ) {
  return false;
 }
 $post = get_post( $postid );
 if ( esc_html( $post->post_title ) !== esc_html( stripslashes( $name ) ) ) {
  return false;
 }
 return ( $passcode == get_post_meta( $postid, 'passcode', true ) );
}

function djv_engarde_submit_orders() {
 global $wpdb;
 
 status_header(200);
 if ( isset( $_REQUEST['nonce'] ) ) {
  wp_verify_nonce('submit_orders', 'nonce');
  if ( isset( $_REQUEST['passcode'] ) && isset( $_REQUEST['postid'] ) && isset( $_REQUEST['characterName'] ) && djv_engarde_verify_passcode( $_REQUEST['postid'], $_REQUEST['characterName'], $_REQUEST['passcode']) ) {
   if ( isset( $_REQUEST['month'] ) && 
        isset( $_REQUEST['support'] ) && 
        isset( $_REQUEST['support-effect'] ) &&
        isset( $_REQUEST['regiment'] ) &&
        isset( $_REQUEST['regiment-effect'] ) &&
        isset( $_REQUEST['rank'] ) &&
        isset( $_REQUEST['rank-effect'] ) &&
        isset( $_REQUEST['club'] ) &&
        isset( $_REQUEST['club-effect'] ) &&
        isset( $_REQUEST['location'] ) &&
        isset( $_REQUEST['location-notes'] ) &&
        isset( $_REQUEST['location-effect'] ) &&
        isset( $_REQUEST['week1'] ) &&
        isset( $_REQUEST['week1-notes'] ) &&
        isset( $_REQUEST['week1-effect'] ) &&
        isset( $_REQUEST['week2'] ) &&
        isset( $_REQUEST['week2-notes'] ) &&
        isset( $_REQUEST['week2-effect'] ) &&
        isset( $_REQUEST['week3'] ) &&
        isset( $_REQUEST['week3-notes'] ) &&
        isset( $_REQUEST['week3-effect'] ) &&
        isset( $_REQUEST['week4'] ) &&
        isset( $_REQUEST['week4-notes'] ) &&
        isset( $_REQUEST['week4-effect'] ) &&
        isset( $_REQUEST['other-notes'] ) &&
        isset( $_REQUEST['other-effect'] ) ) {
     if ( get_option ( 'djv_engarde_options' ) ['closed'] ) {
      echo '<p>Submission of orders for ' . esc_html ( stripslashes ( $_REQUEST ['month'] ) ) . ' ihave now closed as the orders are being processed and the actions for the month resolved. Please awit the publication of the newspaper and then submit orders for the following month.</p>';
     } else {
      $table_name = $wpdb->prefix . 'engarde_orderrows';
      $month_name = remove_accents( $_REQUEST['month'] );
      
      $rows = array( array( 'key' =>   10, 'text' => 'Support',
                            'action' => esc_html( stripslashes( $_REQUEST['support'] ) ),
                            'notes' => '',
                            'effect' => esc_html( stripslashes( $_REQUEST['support-effect'] ) ) ),
                     array( 'key' =>   20, 'text' => 'Regiment',
                            'action' => esc_html( stripslashes( $_REQUEST['regiment'] ) ),
                            'notes' => '',
                            'effect' => esc_html( stripslashes( $_REQUEST['regiment-effect'] ) ) ),
                     array( 'key' =>   30, 'text' => 'Rank',
                            'action' => esc_html( stripslashes( $_REQUEST['rank'] ) ),
                            'notes' => '',
                            'effect' => esc_html( stripslashes( $_REQUEST['rank-effect'] ) ) ), 
                     array( 'key' =>   40, 'text' => 'Club',
                            'action' => esc_html( stripslashes( $_REQUEST['club'] ) ),
                            'notes' => '',
                            'effect' => esc_html( stripslashes( $_REQUEST['club-effect'] ) ) ),
                     array( 'key' =>   50, 'text' => 'Location',
                            'action' => esc_html( stripslashes( $_REQUEST['location'] ) ),
                            'notes' => esc_html( stripslashes( $_REQUEST['location-notes'] ) ),
                            'effect' => esc_html( stripslashes( $_REQUEST['location-effect'] ) ) ),
                     array( 'key' =>  100, 'text' => 'Week 1',
                            'action' => esc_html( stripslashes( $_REQUEST['week1'] ) ),
                            'notes'  => esc_html( stripslashes( $_REQUEST['week1-notes'] ) ),
                            'effect' => esc_html( stripslashes( $_REQUEST['week1-effect'] ) ) ),
                     array( 'key' =>  200, 'text' => 'Week 2',
                            'action' => esc_html( stripslashes( $_REQUEST['week2'] ) ),
                            'notes'  => esc_html( stripslashes( $_REQUEST['week2-notes'] ) ),
                            'effect' => esc_html( stripslashes( $_REQUEST['week2-effect'] ) ) ),
                     array( 'key' =>  300, 'text' => 'Week 3',
                            'action' => esc_html( stripslashes( $_REQUEST['week3'] ) ),
                            'notes'  => esc_html( stripslashes( $_REQUEST['week3-notes'] ) ),
                            'effect' => esc_html( stripslashes( $_REQUEST['week3-effect'] ) ) ),
                     array( 'key' =>  400, 'text' => 'Week 4',
                            'action' => esc_html( stripslashes( $_REQUEST['week4'] ) ),
                            'notes'  => esc_html( stripslashes( $_REQUEST['week4-notes'] ) ),
                            'effect' => esc_html( stripslashes( $_REQUEST['week4-effect'] ) ) ),
                     array( 'key' => 1000, 'text' => 'Other Notes',
                            'action' => '',
                            'notes'  => esc_html( stripslashes( $_REQUEST['other-notes'] ) ),
                            'effect' => esc_html( stripslashes( $_REQUEST['other-effect'] ) ) )
              );
      
      $text .= '<table>' . PHP_EOL;
      $text .= '<tr><td>Name</td><td>' . esc_html( stripslashes( $_REQUEST['characterName'] ) ) . '</td><td></td></tr>' . PHP_EOL;
      $text .= '<tr><td>Month</td><td>' . esc_html( stripslashes( $_REQUEST['month'] ) ) . '</td><td></td></tr>' . PHP_EOL;
      
      foreach($rows as $row) {
       $wpdb->replace($table_name, array( 'page_id' => $_REQUEST['postid'],
         'month' => $month_name,
         'rowid' => $row['key'],
         'action' => $row['action'],
         'notes' => $row['notes'],
         'effect' => $row['effect'],
         'version' => '3.26.0') );
        
       if($wpdb->last_error !== '') {
        $str   = htmlspecialchars( $wpdb->last_error, ENT_QUOTES );
        $query = htmlspecialchars( $wpdb->last_query, ENT_QUOTES );
        echo "<div id='error'><p class='wpdberror'><strong>WordPress database error:</strong> [" . $str . "]<br /><code>" . $query . "</code></p></div>";
       }
       
       if ($row['notes'] !== '') {
        $text .= '<tr><td>' . $row['text'] . '</td><td>' . $row['action'] . '<p>' . $row['notes'] . '</p></td><td>' . $row['effect'] . '</tr>';
       } else {
        $text .= '<tr><td>' . $row['text'] . '</td><td>' . $row['action'] . '</td><td>' . $row['effect'] . '</tr>';
       }
      }
      
      update_post_meta( $_REQUEST['postid'], 'lastordersfor', esc_html( stripslashes( $_REQUEST['month'] ) ) );
      date_default_timezone_set("Europe/London");
      update_post_meta( $_REQUEST['postid'], 'lastorderson', date( '\\o\\n l jS \\of F Y \\a\\t h:i:s A' ) );
      
      $text .= '</table>';
      $text .= '<p>Orders recorded by En Garde Software Version 3.26.0</p>';
      $to = get_option('djv_engarde_options')['email'];
      $cc = "";
      if (!empty( $_REQUEST['ccemail'] ) ) {
       $cc = esc_html( $_REQUEST['ccemail'] );
      } else if ( !empty( get_post_meta( $_REQUEST['postid'], 'email', true ) ) ) {
       $cc = esc_html( get_post_meta( $_REQUEST['postid'], 'email', true ) );
      }
      if ( empty( $cc ) || (!filter_var($cc, FILTER_VALIDATE_EMAIL) ) ) {
       $cc = ""; // Invalid email address
       delete_post_meta( $_REQUEST['postid'], 'email' );
      } else {
       update_post_meta( $_REQUEST['postid'], 'email', $cc );
      }
      if ( !empty( $cc ) ) {
       $to .= ',' . $cc;
      }
      //echo '<pre>' . print_r( get_option('djv_engarde_options') ) . '</pre>';
      $subject = 'En Garde order for ' . esc_html( stripslashes( $_REQUEST['characterName'] ) ) . ' for ' . esc_html( stripslashes( $_REQUEST['month'] ) );
      $headers = array('Content-Type: text/html; charset=UTF-8','From: tremlas <' . get_option('djv_engarde_options')['email'] . '>');
      $sent_mail = wp_mail( $to, $subject, $text, $headers );
      if ( $sent_mail ) {
       if ( empty( $cc ) ) {
        echo '<p>The following En Garde order has been submitted:</p>';
       } else {
        echo '<p>The following En Garde order has been submitted and copied to ' . $cc . ':</p>';
       }
      } else {
       echo '<p><b>An error occurred sending your En Garde order to ' . $to . '.</b> The orders have been saved in the database and were: ';
      }
      echo $text;
      if ( $sent_mail  && (! empty( $cc) ) ) {
       echo '<p><b>Note:</b> The email could take a little while to arrive (I have seen delays of a few hours)</p>';
      }
     }
    } else {
     echo '<p>Something went wrong with your En Garde order submission. The technical details received were:</p>';
     echo '<p><pre>' . print_r( $_REQUEST ) . '</pre></p>';
    }
  } else {
   echo '<p>Your passcode did not match that recorded for the character for which you are trying to submit orders.</p>';
  }
  if ( isset( $_REQUEST['_wp_http_referer'] ) ) {
   echo '<p>Return to <a href="' . $_REQUEST['_wp_http_referer'] .'">';
   if ( isset( $_REQUEST['characterName'] ) ) {
    echo esc_html( stripslashes( $_REQUEST['characterName'] ) ) . '\'s ';
   } 
   echo 'character sheet</a>.</p>';
  }
 }
 die();
 //request handlers should die() when they complete their task
}
add_action( 'admin_post_nopriv_djv_engarde_submit_orders', 'djv_engarde_submit_orders' );
add_action( 'admin_post_djv_engarde_submit_orders', 'djv_engarde_submit_orders' );

// show wp_mail() errors
//function djv_engarde_onMailError( $wp_error ) {
// echo "<pre>";
// print_r($wp_error);
// echo "</pre>";
//}add_action( 'wp_mail_failed', 'djv_engarde_onMailError', 10, 1 );

function djv_engarde_engardeListOrders_shortcode( $attr, $content = null ) {
 global $wpdb;
 $text = '';
 if ( current_user_can( 'edit_others_pages' ) ) {
  if ( isset($attr) && isset( $attr['month'] ) ) {
   $month = remove_accents( $attr['month'] );
  } else {
   $month = remove_accents( get_option('djv_engarde_options')['month'] );
   $text .= '<form action="' . esc_url( admin_url('admin-post.php')) . '" method="post"/>' . PHP_EOL;
   $text .= wp_nonce_field('list_orders', 'nonce', true, false) . PHP_EOL;
   $text .= '<input type="hidden" name="action" value="djv_engarde_process_do_it"/>' . PHP_EOL;
   $text .= '<input type="hidden" name="month" value="' . $month . '"/>' . PHP_EOL;
   $text .= '<input type="submit" value="Process orders for current turn (update weapon expertise)"/>' . PHP_EOL;
   $text .= '</form>' . PHP_EOL;
  }
  $table_name  = $wpdb->prefix . 'engarde_orderrows';
  $rowname_tbl = $wpdb->prefix . 'engarde_rownames';
  
  $postids=$wpdb->get_col(
	"SELECT      ID
	 FROM        $wpdb->posts
	 WHERE       ID IN (SELECT page_id FROM $table_name WHERE month = '$month')" );
  
  if($wpdb->last_error !== '') {
   $str   = htmlspecialchars( $wpdb->last_error, ENT_QUOTES );
   $query = htmlspecialchars( $wpdb->last_query, ENT_QUOTES );
   $text .= "<div id='error'><p class='wpdberror'><strong>WordPress database error:</strong> [" . $str . "]<br /><code>" . $query . "</code></p></div>";
  }
  
  if ($postids) {
   $args = array(
    'post_type' => 'page',
    'post__in' => $postids,
    'orderby' => 'title',
    'order' => 'ASC',
    'nopaging' => true
   );
   $query = new WP_Query($args);
   if ($query->have_posts()) {
    while( $query->have_Posts() ) {
     $query->the_post();
     $text .= '<hr><h3><a href="' . get_permalink() . '">' . get_the_title() . '</a></h3>' . PHP_EOL;
     $text .= '<table>';
     $id = get_the_ID();
     $details = $wpdb->get_results("SELECT R.name, O.action, O.notes, O.effect 
                                      FROM $table_name AS O INNER JOIN $rowname_tbl AS R 
                                     WHERE O.month = '$month' 
                                       AND O.page_id = $id 
                                       AND O.rowid = R.id 
                                  ORDER BY O.rowid ASC" );
     foreach($details as $detail) {
      $text .= '<tr><td>' . $detail->name . '</td><td>' . $detail->action . '<p>' . $detail->notes . '</p></td><td>' . $detail->effect . '</td></tr>' . PHP_EOL;
     }
     $text .= '</table>' . PHP_EOL;
    }
    wp_reset_postdata();
   }
  }
 } else {
  $text = '<p>Current user cannot edit others pages, so not showing the orders!</p>';
 }
 return $text;
}
add_shortcode( 'enGardeLastOrders', 'djv_engarde_engardeListOrders_shortcode' );

function djv_engarde_process_do_it() {
 global $wpdb;
 status_header(200);
 if ( isset( $_REQUEST['nonce'] ) ) {
  wp_verify_nonce('list_orders', 'nonce');
  
  if ( isset( $_REQUEST['month']) ) {
   $month = $_REQUEST['month'];
   $table_name  = $wpdb->prefix . 'engarde_orderrows';
   $rowname_tbl = $wpdb->prefix . 'engarde_rownames';
  
   $details = $wpdb->get_results("SELECT O.page_id, R.name, O.action
                                    FROM $table_name AS O INNER JOIN $rowname_tbl AS R
                                    WHERE O.month = '$month'#
                                      AND O.action LIKE 'Practice%'
                                      AND O.rowid IN ('100','200','300','400')
                                      AND O.rowid = R.id
                                 ORDER BY O.page_id ASC, O.rowid ASC" );
   foreach($details as $detail) {
    $words = explode(' ',$detail->action);
    if ( $words[1] ) {
     $skill = get_post_meta( intval( $detail->page_id ), $words[1], true );
     if ( $skill ) {
      $values = explode('.',$skill);
      $values[1] = $values[1] + 1;
      if ($values[1] == 4) {
       $values[0] = $values[0] + 1;
       $values[1] = 0;
      }
      $skill = $values[0] . '.' . $values[1];
      update_post_meta( intval( $detail->page_id ), $words[1], $skill );
      echo '<p>' . get_the_title( intval( $detail->page_id ) ) . '\'s ' . $words[1] . ' skill has been updated to ' . $skill . '.</p>';
     } else {
      echo '<p>Cannot update practise for ' . get_the_title( intval( $detail->page_id ) ) . ' as the action of <q>' . $detail->action . '</q> did not have a second word.</p>';
     }
    } else {
     echo '<p>Cannot update practise for ' . get_the_title( intval( $detail->page_id ) ) . ' as he does not have metadata to match the action of <q>' . $detail->action . '</q>.</p>';
    }
   }
  } else {
   echo '<p>No month field received - cannot process orders!</p>';
  }
 } else {
  echo '<p>No nonce field - so no processing</p>'; 
 }
 die();
 //request handlers should die() when they complete their task
}
add_action( 'admin_post_nopriv_djv_engarde_process_do_it', 'djv_engarde_process_do_it' );
add_action( 'admin_post_djv_engarde_process_do_it', 'djv_engarde_process_do_it' );

?>