/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Math*/
/* exported Club */

function Club(name,requirementFunction,failFunction,dues,sp,limit,divisor) {
    this._name = name;
    this._rf = requirementFunction;
    this._failf = failFunction;
    this._dues = dues;
    this._sp = sp;
    this._limit = limit;
    this._divisor = divisor;
}

Club.prototype = {
    get name() {
        return this._name;
    },
    get dues() {
        return this._dues;
    },
    get sp() {
        return this._sp;
    },
    get limit() {
        return this._limit;
    },
    get divisor() {
        return this._divisor;
    },
    meetsRequirement : function(regiment,sl) {
        return this._rf(regiment,sl);
    },
    failsRequirement : function(regiment,sl) {
        return this._failf(this._name,regiment,sl);
    }
};

Club._socialLevelRequirementFunction = function(reqsl) {
    return function(regiment,sl) {
        return (sl >= reqsl);
    };
};
Club._socialLevelFailFunction = function(reqsl) {
    return function(name,regiment,sl) {
        return "Your social level of "+sl+" does not meet the minimum of "+reqsl+" for "+name+".";
    };
};
Club._horseGuardsRequirementFunction = function(regiment,sl) {
    return (regiment ? regiment.isHorseGuards : false);
};
Club._horseGuardsFailFunction = function(name,regiment,sl) {
    if (regiment) {
        return "Your regiment, "+regiment+", is not in the Horse Guards Brigade (the Dragoon Guards and the Queen's Own Carabiniers make up the Horse Guards Brigade).";
    } else {
        return "Application refused unless you succeed in an application to join the Dragoon Guards or the Queen's Own Carabiniers.";
    }
};

Club._clubs = [];
Club.addClub = function(name,requirementFunction,failFunction,dues,sp,limit,divisor) {
    Club._clubs.push(new Club(name,requirementFunction,failFunction,dues,sp,limit,divisor));
};
Club.forEach = function(f) {
    Club._clubs.forEach(f);
};
Club.forName = function(name) {
    if (!name || name === "") {
        return undefined; // Not throw an exception
    }
    for(var i=0; i<Club._clubs.length; i++) {
        if (Club._clubs[i].name === name) {
            return Club._clubs[i];
        }
    }
    throw new Error("Unknown club '"+name+"'");
};
Club.addClub("Bothwell's"      ,Club._socialLevelRequirementFunction(12),Club._socialLevelFailFunction(12),30,8,10000000,500);
Club.addClub("Hunter's"        ,Club._socialLevelRequirementFunction(9) ,Club._socialLevelFailFunction(9) ,20,6,200,     300);
Club.addClub("The Horse Guards",Club._horseGuardsRequirementFunction    ,Club._horseGuardsFailFunction    ,20,4,250     ,300);
Club.addClub("The Blue Gables", Club._socialLevelRequirementFunction(7) ,Club._socialLevelFailFunction(7) ,15,4,150     ,200);
Club.addClub("The Frog & Peach",Club._socialLevelRequirementFunction(5) ,Club._socialLevelFailFunction(5) ,10,3,100     ,150);
Club.addClub("Red Phillips"    ,Club._socialLevelRequirementFunction(3) ,Club._socialLevelFailFunction(3) , 5,2, 50     ,150);
