/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Row, Club */
/* exported ClubRow */

function setupClubs(cid) {
}

function ClubRow(cid,sl,regiment,oldClub) {
    Row.call(this,cid);
    
    this._sl = sl;
    this._regiment = regiment;
    this._oldClub = oldClub;
    
    this._leave = document.getElementById(cid+"-club-leave");
    this._leaveLabel = document.getElementById(cid+"-club-leave-label");
    this._choice = document.getElementById(cid+"-club");
    this._choiceLabel = document.getElementById(cid+"-club-join-label");
   
    if (this._oldClub) {
        this._leaveLabel.textContent = "Leave "+this._oldClub.name;
    }
    
    this._choice.length = 1; // Keep the blank first entry
    this._noEligibleClub = true;
    var self = this;
    Club.forEach(function(club) {
        var option = document.createElement("option");
        option.text = club.name;
        if (club.meetsRequirement(self._regiment,self._sl)) {
            self._noEligibleClub = false;
        } else {
            option.disabled = true;
        }
        self._choice.add(option);
    });

    this.disableOnFlag(this._noEligibleClub,this._choice,this._choiceLabel);
    
    this.controlledBy(this._leave);
    this.controlledBy(this._choice);
    
    this.addInnerHTML(document.getElementById(cid+"-club-effect"));
    this.addOrderField(document.getElementById(cid+"-form-club"));
    this.addNotesField(document.getElementById(cid+"-form-club-effect"));
    
    this.processChange(this,undefined);
}

ClubRow.prototype = Object.create(Row.prototype);

ClubRow.prototype.processChange = function(self,event) {
    var club = (self._leave.checked ? null : self._oldClub);
    var order = "";
    var notes = ""; // Default text
    if (self._leave.checked && self._oldClub) {
        order = "Leave "+self._oldClub.name;
    }
    if (self._choice.selectedIndex > 0) {
        club = Club.forName(self._choice.options[self._choice.selectedIndex].text);
        order += (order === "" ? "" : " and ") + "Join "+club.name+".";
        if (club.meetsRequirement(self._regiment,self._sl)) {
            notes = "Joined "+club.name+". ";
        } else {
            notes = club.failsRequirement(self._regiment,self._sl);
            club = null;
        }
    }
    if (self._inParis.checked && club) {
        notes += "Dues of "+club.dues+" Crowns. Gain "+club.sp+" Status Points.";
    } else if (!self._inParis.checked) {
        notes += "Cannot change club while out of Paris (no club fees will be levied however).";
    } else if (self._noEligibleClub) {
        notes += "Sadly no club will accept one with your low social level as a member.";
    }
    if (order === "") {
        order = "No Change"; 
    }
    
    // enable/disable fields as appropriate
    if (self._inParis.checked) {
        self.enableOnFlag(self._oldClub,self._leave,self._leaveLabel);
        if (self._oldClub) {
            self.disableOnFlag(!self._leave.checked,self._choice,self._choiceLabel);
        } else {
            self.enable(self._choice,self._choiceLabel);
        }
    } else {
        self.disable(self._leave,self._leaveLabel);
        self.disable(self._choice,self._choiceLabel);
    }

    self.notifyOrder(order);
    self.notifyCommentary(notes);
};
