/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Row */
/* exported LocationRow */

function LocationRow(cid,location,month,regiment,battle) {
    Row.call(this,cid);
    
    this._location = location;
    this._month = month;
    this._regiment = regiment;
    this._battle = battle;
       
    this._inParisLabel = document.getElementById(cid+"-location-paris-label");
    this._atWarLabel = document.getElementById(cid+"-location-at-war-label");
    this._atFrontierLabel = document.getElementById(cid+"-location-frontier-label");
    this._locationOtherLabel = document.getElementById(cid+"-location-other-label");
    this._battleOutcome = document.getElementById(cid+"-battle-outcome");
    this._bravery = document.getElementById(cid+"-bravery");
    this._braveryText = document.getElementById(cid+"-bravery-description");
    
    this._text = document.getElementById(cid+"-location-notes");
    this.controlledBy(this._text);

    // Is it time to resolve battles?
    this._resolveBattle = false;
    if (this._location === 'At War') {
        if ("/septembre/".indexOf("/"+this._month.split(" ")[0]+"/") !== -1) {
            this._resolveBattle = true;
        }
    } else if (this._location === 'At Frontier') {
        if ("/mars/juin/septembre/décembre/".indexOf("/"+this._month.split(" ")[0]+"/") !== -1) {
            this._resolveBattle = true;
        }
    }
    
    if (this._resolveBattle && this._battle) {
        this._death = 2;
        this._mention = 12;
        this._promotion = 12;
        this._crowns = "-";
        var self = this;
        this._battle.split(" ").forEach(function(sub){
            if (sub.slice(0,1) === "D") {
                self._death = Number(sub.slice(1));
            } else if (sub.slice(0,1) === "M") {
                self._mention = Number(sub.slice(1));
            } else if (sub.slice(0,1) === "P") {
                self._promotion = Number(sub.slice(1));
            } else if (sub.slice(0,1) === "C") {
                self._crowns = Number(sub.slice(1));
            } else if (sub.indexOf("x") !== -1) {
                self._plunder = sub.split("x")[1] + "x" + sub.split("x")[0]+"d6";
            }
        });
        
        this._bravery.min = -13 + this._death;
        this._bravery.max = this._death - 3;
        this._bravery.value = 0;
        var options = "";
        for(var i=this._bravery.min; i<=this._bravery.max; i++) {
            options += '<option value="'+i+'"/>';
        }
        document.getElementById(cid+"-bravery-tickmarks").innerHTML = options;
        
        this.show(this._battleOutcome);
        
        document.getElementById(cid+"-decrease-bravery").onclick = function() {
            var newvalue = self._bravery.valueAsNumber-1;
            if (newvalue >= self._bravery.min) {
                self._bravery.value = newvalue;
                self.processChange(self,undefined);
            }
        };
        document.getElementById(cid+"-increase-bravery").onclick = function() {
            var newvalue = self._bravery.valueAsNumber+1;
            if (newvalue <= self._bravery.max) {
                self._bravery.value = newvalue;
                self.processChange(self,undefined);
            }
        };
        
        this.controlledBy(this._bravery);
    } else {
        this.hide(this._battleOutcome);
    }
    
    this.addInnerHTML(document.getElementById(cid+"-location-effect"));
    this.addOrderField(document.getElementById(cid+"-form-location"));
    this.addDetailsField(document.getElementById(cid+"-form-location-notes"));
    this.addNotesField(document.getElementById(cid+"-form-location-effect"));
    
    // Default to other if the current location is not a standard one
    if (this._location !== "In Paris" && this._location !== "At War" && this._location !== "At Frontier") {
        this._locationOther.checked = true;
    }
    
    this.processChange(this,undefined);
}

LocationRow.prototype = Object.create(Row.prototype);
LocationRow.prototype.show = function(element) {
    element.style.display='';
};
LocationRow.prototype.hide = function(element) {
    element.style.display='none';
};

LocationRow.prototype.processChange = function(self,event) {
    var order = "Unknown";
    var details = "";
    var notes = ""; // Default text

    if ( (self._location === 'At War') || (self._location === 'At Frontier') ) {
        self.disable(self._inParis,self._inParisLabel);
        self.disable(self._atWar,self._atWarLabel);
        self.disable(self._atFrontier,self._atFrontierLabel);
        self.disable(self._locationOther,self._locationOtherLabel);
    } else {
        if ("/juin/juillet/aôut/".indexOf("/"+self._month.split(" ")[0]+"/") === -1) {
        // Not the summer months, so cannot join its regiment at war
            self.disable(self._atWar,self._atWarLabel);
        }
        self.enable(self._inParis,self._inParisLabel);
        self.enable(self._atFrontier,self._atFrontierLabel);
        self.enable(self._locationOther,self._LocationOtherLabel);
    }
    if (self._location === 'At War') {
        self._atWar.checked = true;
    } else if (self._location === 'At Frontier') {
        self._atFrontier.checked = true;
    }
    
    if (self._inParis.checked) {
        order = "In Paris";
    } else if (self._atWar.checked) {
        order = "Gone with Regiment";
        notes = "You must remain with your regiment until the end of the campaigning season.";
    } else if (self._atFrontier.checked) {
        order = "Gone with the Frontier Regiment";
        notes = "You must remain with the frontier regiment until the end of the campaigning season. Frontier regiments are independent of the regular army and you do not need to be a member of any other regiment to volunteer to join the frontier regiments for a season (and you can still volunteer even if you are in another regiment - you will be given a leave of absence).";
    } else if (self._locationOther.checked) {
        order = "Out of Paris";
        notes = "You may return next month if desired.";
    }
    
    if (self._resolveBattle) {
        var adjective;
        if (self._bravery.valueAsNumber > 4) {
            adjective = "Recklessly Brave";
        } else if (self._bravery.valueAsNumber === 4) {
            adjective = "Courageous";
        } else if (self._bravery.valueAsNumber === 3) {
            adjective = "Audiacious";
        } else if (self._bravery.valueAsNumber === 2) {
            adjective = "Valiant";
        } else if (self._bravery.valueAsNumber === 1) {
            adjective = "Brave";
        } else if (self._bravery.valueAsNumber === 0) {
            adjective = "Stoic";
        } else if (self._bravery.valueAsNumber === -1) {
            adjective = "Timid";
        } else if (self._bravery.valueAsNumber === -2) {
            adjective = "Craven";
        } else if (self._bravery.valueAsNumber === -3) {
            adjective = "Cowardly";
        } else if (self._bravery.valueAsNumber === -4) {
            adjective = "Dastardly";
        } else {
            adjective = "Abjectly Cowardly";
        }
        
        self._braveryText.innerHTML = adjective +" ("+self._bravery.value+")";
            
        order += ", apply a bravery adjustment of "+self._bravery.value;
        notes = "You will die in the battle if you roll "+(self._death-self._bravery.valueAsNumber)+" or more on 2d6. ";
        if (self._bravery.valueAsNumber >= 0) {
            notes += "You will be mentioned in dispatches on a roll of "+(self._mention-self._bravery.valueAsNumber)+" or more on 2d6. ";
            if (self._regiment) {
                notes += "You will be promoted if there is an opening on a roll of "+(self._promotion-self._bravery.valueAsNumber)+" or more on 2d6. ";
            }
            if (self._crowns !== "-") {
                notes += "You will receive "+self._plunder+" crowns in plunder on a roll of "+(Number(self._crowns)-self._bravery.valueAsNumber)+" or more on 2d6. ";
            }
        } else {
            notes += "Your cowardice will be noticed on a roll of 3 or more on 1d6 and if so you will lose "+(0-self._bravery.valueAsNumber*2)+" status points once back in Paris. ";
            if (self._regiment) {
                notes += "If the roll is 6 you will also be dismissed from your regiment and can only join a frontier regiment until you have been mentioned in dispatches. ";
            }
            notes += "If your cowardice is not detected, you will be mentioned in dispatches on a roll of "+self._mention+" or more on 2d6. ";
            if (self._regiment) {
                notes += "and promoted if there is an opening in your regiment on a roll of "+(self._promotion-self._bravery.valueAsNumber)+" or more on 2d6. ";
            }
        }
    }
    
    var i;
    if (!self._inParis.checked) {
        for(i=1; i<5; i++) {
            self.disable(document.getElementById(self._cid+"-week-"+i),document.getElementById(self._cid+"-week-"+i+"-heading"));
            self.disable(document.getElementById(self._cid+"-week-"+i+"-notes"),document.getElementById(self._cid+"-week-"+i+"-notes-label"));
        }
    } else {
        for(i=1; i<5; i++) {
            self.enable(document.getElementById(self._cid+"-week-"+i),document.getElementById(self._cid+"-week-"+i+"-heading"));
            self.enable(document.getElementById(self._cid+"-week-"+i+"-notes"),document.getElementById(self._cid+"-week-"+i+"-notes-label"));
        }
    }

    
    self.notifyOrder(order);
    self.notifyDetails(self._text.value);
    self.notifyCommentary(notes);
};
