/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals alert, define, exports, esx */

/* Tell users that aren't providing ECMAScript 5 (or better) that it probably won't work :) */
// From: https://codereview.stackexchange.com/questions/147892/small-javascript-library-for-ecmascript-version-detection
// License: cc-by-sa 3.0
(function(root, name, factory) {
    'use strict';

    if (("function" === typeof define) && define.amd) {
        define([ exports ], factory);
    }
    else if (typeof exports === 'object') {
        factory(exports);
    }
    else {
        factory(root[name] = {});
    }
})(this, 'esx', function(exports) {
    'use strict';

    var features = {
        "arrayComprehensions" : "[for(_ of [0])_]",
        "arrowFunction" : "(_=>_)",
        "class" : "(class{})",
        "const" : "const c=true",
        "defaultParams" : "(function(a=false){})",
        "destructuring" : "let {d}={a:true}",
        "forOf" : "for(var b of [])",
        "generator" : "(function*(){})",
        "getter" : "({get a(){}})",
        "label" : "l:0",
        "let" : "let o",
        "reservedWords" : "({catch:true})",
        "setter" : "({set a(v){}})",
        "spread" : "[...[]]",
        "stringInterpolation" : "`$\{0}`",
        "stringLineBreak" : "'\\\n'",
        "super" : "({b(){super.a}})",
        "yield" : "(function*(){yield true})"
    };

    // jshint evil:true
    function evaluate(code) {
        try {
            eval(code);
            return true;
        } catch (e) {
            return false;
        }
    }

    /**
     * Check if a set of features are supported.
     */
    function supports() {
        var code = "(function(){";
        var i = 0, len = arguments.length;
        for (; i < len; ++i) {
            var feature = arguments[i].toString();
            if (features.hasOwnProperty(feature)) {
                code += features[feature] + ';';
            }
        }

        code += "})()";
        return evaluate(code);
    }

    exports.supports = supports;

    function checkES7() {
        return supports("arrayComprehensions");
    }

    function checkES6() {
        var methods = 'function' === typeof Object.assign && 'function' === typeof Object.freeze;

        var syntax = supports("arrowFunction", "class", "const", "forOf",
                "defaultParams", "destructuring", "super", "yield");

        return methods && syntax;
    }

    function checkES5() {
        var methods = 'function' === typeof [].filter && 'function' === typeof Function.prototype.bind && 'function' === typeof Object.defineProperty && 'function' === typeof ''.trim && 'object' === typeof JSON;

        var syntax = supports("reservedWords");
        return methods && syntax;
    }

    function checkES3() {
        return "function" === typeof [].hasOwnProperty;
    }

    /**
     * Check for ECMAScript version.
     */
    exports.detectVersion = function() {
        return checkES7() ? 7 : checkES6() ? 6 : checkES5() ? 5
                : checkES3() ? 3 : null;
    };

});
// End of reused code from
// https://codereview.stackexchange.com/questions/147892/small-javascript-library-for-ecmascript-version-detection

if (esx.detectVersion() < 5) {
    alert("The scripts used for the En Garde form valdiation and submission assume that you're using a modern browser. It would seem that you're not using a modern browser. Your mileage in using the form will vary and there's no guarantee I'll see what you submit. I recommend trying with a more up-to-date browser :)");
}

// pseudo-polyfill for Function.getName
if (Function && 
    !Function.prototype.getName) {
    Function.prototype.getName = function() {
        if (!this.name) {
            this.name  = this.toString().match(/function\s*([^(]*)\(/)[1];
        }
        return this.name;
    };
}
