/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Row, Regiment, Rank */
/* exported RankRow */

function RankRow(cid,sl,regiment,oldRank,stable) {
    Row.call(this,cid);
    
    this._sl = sl;
    this._regiment = regiment;
    this._oldRank = oldRank;
    this._stable = stable;
    
    this._leave = document.getElementById(cid+"-regiment-leave");
    this._newReg = document.getElementById(cid+"-regiment");
    this._choice = document.getElementById(cid+"-rank");
    this._choiceLabel = document.getElementById(cid+"-promote-label");
    
    // Grey out lesser ranks
    if (oldRank) {
        for(var i=1; i<this._choice.length; i++) { // Ignore blank first row :)
            var newRank = Rank.forName(this._choice.options[i].text);
            if (newRank.level <= oldRank.level) {
                this._choice.options[i].disabled = true;
            }
        }
    }
    
    this.controlledBy(this._leave);
    this.controlledBy(this._newReg);
    this.controlledBy(this._choice);
    
    this.addInnerHTML(document.getElementById(cid+"-rank-effect"));
    this.addOrderField(document.getElementById(cid+"-form-rank"));
    this.addNotesField(document.getElementById(cid+"-form-rank-effect"));
    
    this.processChange(this,undefined);
}

RankRow.prototype = Object.create(Row.prototype);

RankRow.prototype.processChange = function(self,event) {
    var horseCount = function() {
        if (self._stable) {
            return Number(self._stable.match(/\d+/)[0]);
        } else {
            return 0;
        }
    };
    var order;
    var notes = "Assuming ";
    if (self._inParis) {
        if (self._regiment || (self._newReg.selectedIndex > 0)) {
            self.enable(self._choice,self._choiceLabel);
            
            if (self._choice.selectedIndex > 0) {
                var oldRank = (self._oldRank ? self._oldRank : Rank.Private);
                var newRank = Rank.forName(self._choice.options[self._choice.selectedIndex].text);
                var actual = self._regiment;
                order = "Apply for "+newRank.rank;
                if (self._newReg.selectedIndex > 0) {
                    var newRegiment = self._newReg.options[self._newReg.selectedIndex].text;
                    actual = Regiment.forName(newRegiment);
                    oldRank = Rank.Private;
                    notes += "your application to "+newRegiment+" is successful and assuming";
                }
                notes += "there's is a position available, your promotion to "+newRank.rank+" will cost "+actual.costToPromote(self._oldRank,newRank)+" Crowns.";
                if ((newRank.horses === 1) && horseCount(self._stable) < 1) {
                    notes += " As a "+newRank.rank+" you will also need a horse (each horse costs 100 Crowns and then 3 Crowns per month. If you have any horses you also need to hire a (single) groom at a cost of 2 Crowns per month).";
                }
                if ((newRank.horses > 1) && horseCount(self._stable) < newRank.horses) {
                    notes += " As a "+newRank.rank+" you will also need "+newRank.horses+" horses (each horse costs 100 Crowns and then 3 Crowns per month. If you have any horses you also need to hire a (single) groom at a cost of 2 Crowns per month).";
                }
                notes += " Your salary will increase to "+actual.salary(newRank)+" Crowns from "+actual.salary(self._oldRank)+" Crowns.";
                notes += " Your status points from your regiment will increase to "+actual.sp(newRank)+".";
            } else {
                order = "No Change";
                notes = ""; // No notes if you're not promoting
            }
        } else {
            order ="No change (not in a regiment)";
            notes = "Cannot purchase a promotion without being in a regiment.";
            self.disable(self._choice,self._choiceLabel);
        }
    } else {
        order = "No Change";
        notes = "Cannot change rank while out of Paris.";
        self.disable(self._choice,self._choiceLabel);
    }
    
    self.notifyOrder(order);
    self.notifyCommentary(notes);
};
