/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Math*/
/* exported Rank, Regiment */

function Rank(rank,level,horses) {
    this._rank = rank;
    this._level = level;
    this._horses = horses;
}

Rank.prototype = {
    get rank() {
        return this._rank;
    },
    get level() {
        return this._level;
    },
    get horses() {
        return this._horses;
    }
};

Rank.Private = new Rank("Private",0,0);
Rank.Subaltern = new Rank("Subaltern",1,0);
Rank._ranks = [Rank.Private, Rank.Subaltern];
Rank.addRank = function(rank,level,horses) {
    Rank._ranks.push(new Rank(rank,level,horses));
};
Rank.forEach = function(f) {
    Rank._ranks.forEach(f);
};
Rank.forName = function(rank) {
    if (!rank || rank === "") {
        return undefined; // Not throw an exception
    }
    for(var i=0; i<Rank._ranks.length; i++) {
        if (Rank._ranks[i].rank === rank) {
            return Rank._ranks[i];
        }
    }
    throw new Error("Unknown rank '"+name+"'");
};

Rank.addRank("Captain",2,1);
Rank.addRank("Major",3,3);
Rank.addRank("Lieutenant Colonel",4,3);
Rank.addRank("Colonel",5,3);

function Regiment(name,isCavalry,weapon,column,guards) {
    this._name = name;
    this._isCavalry = isCavalry;
    this._weapon = weapon;
    this._column = column-1;
    this._guards = (guards ? true : false);
}

Regiment.prototype = {
    get name() {
        return this._name;
    },
    get isCavalry() {
        return this._isCavalry;
    },
    get weapon() {
        return this._weapon;
    },
    get basesl() {
        return [7,6,5,4,3,2,0,0,0,0][this._column];
    },
    get isHorseGuards() {
        return this._guards;
    },
    salary: function(rank) {
        // Basic calculation
        var salary = 12 - this._column + 4*rank.level;
        // Corrections!
        if (this._column === 1) { salary -= 1; }
        if (this._column === 3) { salary -= 1; }
        if (this._column > 4) { salary -= 1; }
        if (this._column === 6) { salary -= 1; }
        if (this._column === 8) { salary -= 1; }
        
        if (rank._level > 1) { salary -= 2; }
        if ( (this._column > 2) && (rank._level === 1) ) { salary -= 2; }
        
        return salary;
    },
    sp: function(rank) {
        var sp = 6 - this._column + rank.level;
        if (this._column > 3) { sp++; }
        return ((sp > 0) ? sp : 0);
    },
    joinroll: function(sl) {
        return Math.floor((18-sl-this._column)/2);
    },
    _costToPromote: function(oldlevel,newlevel) {
        if (newlevel <= oldlevel) {
            return 0;
        } else if (newlevel > oldlevel+1) {
            return this._costToPromote(oldlevel,oldlevel+1) + this._costToPromote(oldlevel+1,newlevel);
        } else {
            if (oldlevel < 2) {
                return 140 - 10*this._column + oldlevel*10;
            } else if (oldlevel < 4) {
                return 130 - 10*this._column + oldlevel*20;
            } else {
                return 240 - 10*this._column;
            }
        }
    },
    costToPromote: function(oldrank,newrank) {
        return this._costToPromote(oldrank.level,newrank.level);
    }
};

Regiment._regiments = [];
Regiment.addRegiment = function(name,isCavalry,weapon,column,guards) {
    Regiment._regiments.push(new Regiment(name,isCavalry,weapon,column,guards));
};
Regiment.forEach = function(f) {
    Regiment._regiments.forEach(f);
};
Regiment.forName = function(name) {
    if (!name || name === "") {
        return undefined; // Not throw an exception
    }
    for(var i=0; i<Regiment._regiments.length; i++) {
        if (Regiment._regiments[i].name === name) {
            return Regiment._regiments[i];
        }
    }
    throw new Error("Unknown regiment '"+name+"'");
};
Regiment.addRegiment("Royal Foot Guards",               false, "Rapier",  1);
Regiment.addRegiment("Cardinal's Guard",                false, "Rapier",  2);
Regiment.addRegiment("King's Musketeers",               false, "Rapier",  2);
Regiment.addRegiment("Dragoon Guards",                  true,  "Sabre",   3, true);
Regiment.addRegiment("Queen's Own Carabiniers",         true,  "Sabre",   4, true);
Regiment.addRegiment("Archduke Leopold Cuirassiers",    true,  "Sabre",   4);
Regiment.addRegiment("Crown Prince Cuirassiers",        true,  "Sabre",   5);
Regiment.addRegiment("Royal Marines",                   false, "Cutlass", 6);
Regiment.addRegiment("Grand Duke Maximillian Dragoons", true,  "Sabre",   6);
Regiment.addRegiment("Picardy Musketeers",              false, "Rapier",  7);
Regiment.addRegiment("13th Fusiliers",                  false, "Rapier",  8);
Regiment.addRegiment("Princess Louisa Light Dragoons",  true,  "Sabre",   8);
Regiment.addRegiment("53rd Fusiliers",                  false, "Rapier",  9);
Regiment.addRegiment("27th Musketeers",                 false, "Rapier",  9);
Regiment.addRegiment("4th Arquebusiers",                false, "Rapier",  9);
Regiment.addRegiment("69th Arquebusiers",               false, "Rapier",  10);
Regiment.addRegiment("Gascon Regiment",                 false, "Rapier",  10);
Regiment._regiments.sort(function(a, b) {
    return (a._column - b._column);
});
