/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Row, Regiment, Rank */
/* exported RegimentRow */

function RegimentRow(cid,sl,oldRegiment,oldRank,stable) {
    Row.call(this,cid);
    
    this._sl = sl;
    this._oldRegiment = oldRegiment;
    this._oldRank = oldRank;
    this._stable = stable;
    
    this._leave = document.getElementById(cid+"-regiment-leave");
    this._leaveLabel = document.getElementById(cid+"-regiment-leave-label");
    this._choice = document.getElementById(cid+"-regiment");
    this._choiceLabel = document.getElementById(cid+"-regiment-join-label");
   
    if (this._oldRegiment) {
        this._leaveLabel.textContent = "Leave "+this._oldRegiment.name;
    }
    
    this._choice.length = 1; // Keep the blank first entry
    var self = this;
    Regiment.forEach(function(regiment) {
        var option = document.createElement("option");
        option.text = regiment.name;
        option.disabled = (self._sl < regiment.basesl);
        self._choice.add(option);
    });
    
    this.controlledBy(this._leave);
    this.controlledBy(this._choice);
    
    this.addInnerHTML(document.getElementById(cid+"-regiment-effect"));
    this.addOrderField(document.getElementById(cid+"-form-regiment"));
    this.addNotesField(document.getElementById(cid+"-form-regiment-effect"));
    
    this.processChange(this,undefined);
}

RegimentRow.prototype = Object.create(Row.prototype);

RegimentRow.prototype.processChange = function(self,event) {
    var horseCount = function() {
       if (self._stable) {
            return Number(self._stable.match(/\d+/)[0]);
        } else {
            return 0;
        }
    };
    var newRegiment = (self._leave.checked ? null : self._oldRegiment);
    var order = "";
    var notes = ""; // Default text
    var failed = false;
    var sp;
    if (self._oldRegiment) {
        if (self._leave.checked) {
            order = "Leave "+self._oldRegiment.name;
            notes = "Salary of "+self._oldRegiment.salary(self._oldRank)+" Crowns lost. ";
        } else {
            self._choice.selectedIndex = 0;
        }
    }
    if (self._choice.selectedIndex > 0) {
        newRegiment = Regiment.forName(self._choice.options[self._choice.selectedIndex].text);
        order += (order === "" ? "" : " and ") + "Join "+newRegiment.name;
        if (self._sl >= newRegiment.basesl) {
          notes += "With your social level of "+self._sl+" you will need a roll of "+newRegiment.joinroll(self._sl)+" or more on 1d6 for application to succeed. ";
          if (newRegiment.isCavalry && horseCount() < 1) {
              notes += "As "+newRegiment.name+" is a cavalry regiment you will need to ensure that you own at least one horse if your application succeeds (each horse costs 100 Crowns and then 3 Crowns per month. If you have any horses you also need to hire a (single) groom at a cost of 2 Crowns per month). ";
          }
          var salary = newRegiment.salary(Rank.Private);
          notes += "If successful you have a salary of "+salary+" Crown" + ((salary>1) ? "s" : "") +" as a private";
          if (self._inParis.checked) {
              notes += " and gain";
              sp = newRegiment.sp(Rank.Private);
              if (sp === 1) {
                  notes += " a status point this month.";
              } else {
                  notes += " "+sp+" status points this month.";
              }
          } else {
              notes += ".";
          }
        } else {
          notes += newRegiment.name+" requires its applicants to have a minimum social level of "+newRegiment.basesl+" so your application will fail.";
          failed = true;
        }
    } else if (self._oldRegiment && !(self._leave.checked)) {
        order = "No Change";
        notes = "Salary of "+(self._oldRegiment ? self._oldRegiment.salary(self._oldRank) : 0)+" Crowns and gain";
        sp = self._oldRegiment.sp(self._oldRank);
        if (sp === 1) {
            notes += " a status point this month.";
        } else {
            notes += " "+sp+" status points this month.";
        }
    }
    if (!self._inParis.checked) {
        notes += "Cannot change regiment while out of Paris (you still draw your salary however)";
    }
    if (order === "") {
        order = "No Change";
    }

    // enable/disable fields as appropriate
    if (self._inParis.checked) {
        self.enableOnFlag(self._oldRegiment,self._leave,self._leaveLabel);
        if (self._oldRegiment) {
            self.enableOnFlag(self._leave.checked,self._choice,self._choiceLabel);
        } else {
            self.enable(self._choice,self._choiceLabel);
        }
    } else {
        self.disable(self._leave,self._leaveLabel);
        self.disable(self._choice,self._choiceLabel);
    }

    self.notifyOrder(order);
    self.notifyCommentary(notes);
};
