/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals console, window, alert */
/* exported Row */

function Row(cid) {
    this._cid = cid;
    this._orderListeners = [];
    this._detailsListeners = [];
    this._commentaryListeners = [];
    
    this._inParis = document.getElementById(cid+"-location-paris");
    this._atWar = document.getElementById(cid+"-location-at-war");
    this._atFrontier = document.getElementById(cid+"-location-frontier");
    this._locationOther = document.getElementById(cid+"-location-other");

    this.controlledBy(this._inParis);
    this.controlledBy(this._atWar);
    this.controlledBy(this._atFrontier);
    this.controlledBy(this._locationOther);
}

Row.prototype = {
    addOrderListener: function(f) {
        this._orderListeners.push(f);
    },
    addDetailsListener: function(f) {
        this._detailsListeners.push(f);
    },
    addCommentaryListener : function(f) {
        this._commentaryListeners.push(f);
    },
    notifyOrder: function(order) {
        this._orderListeners.forEach(function(listener) { listener(order); } );
    },
    notifyDetails: function(details) {
        this._detailsListeners.forEach(function(listener) { listener(details); } );
    },
    notifyCommentary: function(commentary) {
        this._commentaryListeners.forEach(function(listener) { listener(commentary); } );
    },
    addInnerHTML: function(element) {
        /* jshint debug: true */
        if (element) {
            this.addCommentaryListener(function(commentary) {
                element.innerHTML = "<p>"+commentary+"</p>";
            });
        } else {
            debugger;
        }
    },
    addOrderField: function(element) {
        /* jshint debug: true */
        if (element) {
            this.addOrderListener(function(order) {
                element.value = order;
            });
        } else {
            debugger;
        }
    },
    addDetailsField: function(element) {
        /* jshint debug: true */
        if (element) {
            this.addDetailsListener(function(details) {
                element.value = details;
            });
        } else {
            debugger;
        }
    },
    addNotesField: function(element) {
        /* jshint debug: true */
        if (element) {
            this.addCommentaryListener(function(commentary) {
                element.value = commentary;
            });
        } else {
            debugger;
        }
    },
    processChange: function(self,event) {
        /* jshint debug: true */
        debugger;
    },
    controlledBy: function(element) {
        /* jshint debug: true */
        var self = this; // Capture this
        element.addEventListener("change", function(event) {
            try {
                self.processChange(self,event);
            } catch (e) {
                debugger;
                if (e.stack) {
                    alert(e + "\n\nStack Trace\n-----------------\n" + e.stack);
                } else {
                    alert(e);
                }
            }
            return false;
        }, false);
    },
    disableOnFlag: function(flag,element,label) {
        if (element) {
            element.disabled = flag;
        }
        if (label) {
            label.style = (flag ? "opacity: 0.5;" : "");
        }
    },
    enableOnFlag: function(flag,element,label) {
        this.disableOnFlag(!flag,element,label);
    },
    disable: function(element,label) {
        this.disableOnFlag(true,element,label);
    },
    enable: function(element,label) {
        this.disableOnFlag(false,element,label);
    },
    disableIfOutOfParis: function(element,label) {
        this.disableOnFlag(!(this._inParis.checked),element,label);
    }
};