/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Row, mistressExpense */
/* exported SupportRow */

function SupportRow(cid,sl,allowance,stable,companion,cnotes) {
    Row.call(this,cid);
    
    this._sl = sl;
    this._allowance = allowance;
    this._stable = stable;
    this._companion = companion;
    this._cnotes = cnotes;
    
    this._none = document.getElementById(cid+"-support-none");
    this._noneLabel = document.getElementById(cid+"-support-none-label");
    this._std = document.getElementById(cid+"-support-std");
    this._stdLabel = document.getElementById(cid+"-support-std-label");
    this._con = document.getElementById(cid+"-support-con");
    this._conLabel = document.getElementById(cid+"-support-con-label");
   
    this.controlledBy(this._none);
    this.controlledBy(this._std);
    this.controlledBy(this._con);
    
    this.addInnerHTML(document.getElementById(cid+"-support-effect"));
    this.addOrderField(document.getElementById(cid+"-form-support"));
    this.addNotesField(document.getElementById(cid+"-form-support-effect"));
    
    this.processChange(this,undefined);
}

SupportRow.prototype = Object.create(Row.prototype);

SupportRow.prototype.processChange = function(self,event) {
    var stableCost = function(stable) {
        if (stable) {
            var horses = Number(stable.match(/\d+/)[0]);
            return horses*3+2;
        } else {
            return 0;
        }
    };
    var order;
    var notes;
    if (self._inParis.checked) {
        self.enable(self._none,self._noneLabel);
        self.enable(self._std,self._stdLabel);
        self.enable(self._con,self._conLabel);
        if (self._none.checked) {
            order = "None";
            notes = "0 Crowns spent of an income of "+self._allowance+" Crowns. Social Level will decrease by one.";
        } else if (self._std.checked) {
            order = "Standard";
            notes = ""+self._sl*2+" Crowns spent of an income of "+self._allowance+" Crowns.";
        } else {
            order = "Conspicious";
            notes = ""+self._sl*3+" Crowns spent of an income of "+self._allowance+" Crowns. 1 Status Point gained.";
        }
    } else {
        order = "Out of Paris";
        notes = "Out of Paris - no support costs - all your income of "+self._allowance+" Crowns will be saved.";
        self._none.checked = true;
        self.disable(self._none,self._noneLabel);
        self.disable(self._std,self._stdLabel);
        self.disable(self._con,self._conLabel);
        
    }
    if (self._stable) {
        notes += " Your stable of "+self._stable+" will cost an additional "+stableCost(self._stable)+" Crowns.";
    }
    if (self._companion) {
        var hersl = Number(self._cnotes.match(/\d+/)[0]);
        if (self._cnotes.indexOf("Wealthy") === -1) {
            notes += " Supporting "+self._companion+" will cost an additional "+(hersl*3)+" Crowns to keep her in the manner to which she is accustomed.";
        } else {
            if (hersl > self._sl) {
                notes += " "+self._companion+" makes small gifts to you worth a total of "+(hersl-self._sl)*2+" Crowns.";
            } else {
                notes += " "+self._companion+"'s wealth is able to keep her in the manner to which she is accustomed.";
            }
        }
        var gain = 1;
        if (self._cnotes.indexOf("Stunning") !== -1) {
            gain += 1;
        }
        if (hersl > self._sl) {
            gain += (hersl - self._sl);
        }
        notes += " You gain ";
        if (gain === 1) {
            notes += " a status point";
        } else {
            notes += ""+gain+" status points";
        }
        notes += " for being the companion of "+self._companion+" this month.";
    }

    self.disableIfOutOfParis(self._none,self._noneLabel);
    self.disableIfOutOfParis(self._std,self._stdLabel);
    self.disableIfOutOfParis(self._con,self._conLabel);

    self.notifyOrder(order);
    self.notifyCommentary(notes);
};
