/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Row, Club, setUpClubs, mistressExpense, type, console */
/* exported WeekRow */

function Activity(weekrow,action,detailsId) {
    this._weekrow = weekrow;
    this._action = action;
    this._detailsId = detailsId;
    this._fieldList = [];
}
Activity.prototype = { 
    processActivity: function(self,event) {
        console.log("Base processActivity method called for "+type(this));
        return {order: "", notes: ""};
    },
    selectActivity: function() {
        if (this._detailsId) {
            this.show(this._detailsId);
        }
    },
    deselectActivity: function() {
        if (this._detailsId) {
            this.hide(this._detailsId);
        }
    },
    get fields() {
        return this._fieldList;
    },
    get name() {
        return this._action;
    },
    show: function(suffix) {
        document.getElementById(this._weekrow._prefix+"-"+suffix).style.display='';
    },
    hide: function(suffix) {
        document.getElementById(this._weekrow._prefix+"-"+suffix).style.display='none';
    }
};

function GenericCarouseActivity(weekrow,action,infix) {
    Activity.call(this,weekrow,action,infix+"-details");
    this._infix = infix;
    
    this._detailsCarouse = document.getElementById(this._weekrow._prefix+"-"+infix+"-details-carouse");
    this._detailsCompanion = document.getElementById(this._weekrow._prefix+"-"+infix+"-details-companion");
    this._detailsGamble = document.getElementById(this._weekrow._prefix+"-"+infix+"-details-gamble");
    this._gambleDetailsRounds = document.getElementById(this._weekrow._prefix+"-gamble-details-rounds");
    this._gambleDetailsRoundsLabel = document.getElementById(this._weekrow._prefix+"-gamble-details-rounds-label");
    this._gambleDetailsBet = document.getElementById(this._weekrow._prefix+"-gamble-details-bet");
    this._gambleDetailsBetLabel = document.getElementById(this._weekrow._prefix+"-gamble-details-bet-label");
    this._gambleDetailsCut = document.getElementById(this._weekrow._prefix+"-gamble-details-cut");
    
    this._fieldList.push(this._detailsCarouse,this._detailsCompanion,this._detailsGamble,this._gambleDetailsRounds,this._gambleDetailsBet,this._gambleDetailsCut);
}
GenericCarouseActivity.prototype = Object.create(Activity.prototype);
GenericCarouseActivity.prototype.gambleOrders = function(){
    var plural = function(v,text,labeltext,label) {
        if (v === 1) {
            label.innerHTML = labeltext;
            return v+" "+text;
        } else {
            label.innerHTML = labeltext+"s";
            return v+" "+text+"s";
        }
    };
    var order = plural(this._gambleDetailsRounds.value,"round","Round",this._gambleDetailsRoundsLabel);
    order += " betting "+plural(this._gambleDetailsBet.value,"crown","Crown",this._gambleDetailsBetLabel);
    if (this._gambleDetailsCut.value === "7") {
        order += ", never cutting.";
    } else if (this._gambleDetailsCut.value === "1") {
        order += ", always cutting.";
    } else {
        order += ". cutting on a house roll of "+this._gambleDetailsCut.value+".";
    }
    return order;
};
GenericCarouseActivity.prototype.gambleEffects = function(club){
    var rounds = Number(this._gambleDetailsRounds.value);
    var bet = Number(this._gambleDetailsBet.value);
    var cut = Number(this._gambleDetailsCut.value);
    var losses = Math.round([18, 18, 11, 6, 3, 2, 3, 6][cut]/36*rounds*bet);
    var plural = function(v,word) {
        if (v === 1) {
            return v+" "+word;
        } else {
            return v+" "+word+"s";
        }
    };
    if (!club) {
        return "Expected loss of "+plural(losses,"Crown")+".";
    }
    var sp = Math.round((5+cut)/12*rounds*bet/club.divisor);
    if (sp === 0) {
        return "Expected loss of "+plural(losses,"Crown")+".";
    } else if (sp === 1) {
        return "Expected loss of "+plural(losses,"Crown")+" and gain one status point.";
    } else {
        return "Expected loss of "+plural(losses,"Crown")+" and gain "+sp+" status points.";
    }
};
GenericCarouseActivity.prototype.selectActivity = function(event) {
    Activity.prototype.selectActivity.call(this);
    if (this._detailsGamble.checked) {
        this.show("gamble-details");
    } else {
        this.hide("gamble-details");
    }
};
GenericCarouseActivity.prototype.deselectActivity = function(self,event) {
    Activity.prototype.deselectActivity.call(this);
    this.hide("gamble-details");
};

function ClubActivity(weekrow,cid,sl,club) {
    GenericCarouseActivity.call(this,weekrow,"Visit Club","club");
    this._sl = sl;
    this._initialClub = club;
    this._clubChoice = document.getElementById(cid+"-club");
    this._fieldList.push(this._clubChoice);
    
}
ClubActivity.prototype = Object.create(GenericCarouseActivity.prototype);
Object.defineProperty(ClubActivity.prototype, "_club", { get: function() {
    var actualclub = this._initialClub;
    if (this._clubChoice.selectedIndex > 0) {
        actualclub = Club.forName(this._clubChoice.options[this._clubChoice.selectedIndex].text);
    }
    return actualclub;
}});
ClubActivity.prototype.selectActivity = function(event) {
    if (this._club) {
        GenericCarouseActivity.prototype.selectActivity.call(this);
        this._gambleDetailsBet.max = this._club.limit;
    }
};
ClubActivity.prototype.processActivity = function(self,event) {
    var order, notes;
    if (self._club) {
        order = "Visit "+self._club.name;
        notes = "";
        var cost = 0;
        if (self._detailsCarouse.checked) {
            order += ", carouse";
            notes = " Gain one status point";
            cost = Number(self._sl);
        }
        if (self._detailsCompanion.checked) {
            order += " with companion";
            if (self._cnotes) {
                cost += Number(self._cnotes.match(/\d+/)[0]);
            }
            notes = "Estimated cost of "+cost+" Crowns." + notes;
        } else {
            notes = "Costing "+cost+" Crowns." + notes;
        }
        if (self._detailsGamble.checked) {
            order += " and gamble "+self.gambleOrders();
            notes += " "+self.gambleEffects(self._club);
        }
    } else {
        order = "Attempt to visit club (but not a member of any club!)";
        notes = "Cannot visit a club without being a member of one (if you're attempting to toady with a club member, select other and add the details).";
    }
    
    return {order: order, notes:notes};
};

function BawdyhouseActivity(weekrow,sl) {
    GenericCarouseActivity.call(this,weekrow,"Visit Bawdyhouse","bawdy");
    this._sl = sl;
}
BawdyhouseActivity.prototype = Object.create(GenericCarouseActivity.prototype);
BawdyhouseActivity.prototype.selectActivity = function() {
    GenericCarouseActivity.prototype.selectActivity.call(this);
    this._gambleDetailsBet.max = 10000;
};
BawdyhouseActivity.prototype.processActivity = function(self,event) {
    var order = "Visit Bawdyhouse, carouse";
    var cost = Number(self._sl);
    if (self._detailsCompanion.checked) {
        order += " with bawdyhouse companion";
        cost += Number(self._sl);
    }
    var notes = "Costing "+cost+" Crowns. Gain one status point. There is a 1 in 6 chance of being robbed in the red light district. You may wish to specify how much cash you're carrying when you enter the bawdyhouse (If not specified, it will be assumed that you're bringing 25% of your cash with you).";
    if (self._detailsGamble.checked) {
        order += " and gamble "+self.gambleOrders();
        notes += " "+self.gambleEffects();
    }
    
    return {order: order, notes:notes};
};
function PlayhouseActivity(weekrow,sl) {
    GenericCarouseActivity.call(this,weekrow,"Visit playhouse","play");
    this._sl = sl;
    this._playChoice = document.getElementById(this._weekrow._prefix+"-play");
    this._fieldList.push(this._playChoice);
}
PlayhouseActivity.prototype = Object.create(GenericCarouseActivity.prototype);
PlayhouseActivity.prototype.selectActivity = function() {
    GenericCarouseActivity.prototype.selectActivity.call(this);
    this._gambleDetailsBet.max = 10000;
};
PlayhouseActivity.prototype.processActivity = function(self,event) {
    var order = "Visit playhouse, carouse and watch "+self._playChoice.options[self._playChoice.selectedIndex].text;
    var cost = Number(self._sl);
    var notes;
    if (self._detailsCompanion.checked) {
        order += " with companion";
        if (self._cnotes) {
            cost += Number(self._cnotes.match(/\d+/)[0]);
        }
        notes = "Estimated cost of "+cost+" Crowns.";
    } else {
        notes = "Costing "+cost+" Crowns.";
    }
    notes += " Gain two status points if "+ self._playChoice.options[self._playChoice.selectedIndex].text +" is the most watched play this month.";
    
    return {order: order, notes:notes};
};


function CourtingActivity(weekrow,sl) {
    Activity.call(this,weekrow,"Court new female companion","court-details");
    this._sl = sl;
    this._courtCompanion = document.getElementById(this._weekrow._prefix+"-court-companion");
    // Disable ladies that are of too high a social level - default to the first one that can be chosen
    var defaulted = false;
    for(var i=0; i<this._courtCompanion.options.length; i++) {
        var herSl = Number(this._courtCompanion.options[i].text.match(/\d+/)[0]);
        var diff = this._sl-herSl;
        if (diff < -6) {
            this._courtCompanion.options[i].disabled = true;
        } else if (!defaulted) {
            this._courtCompanion.options[i].defaultSelected = true;
            defaulted = true;
        }
    }
    this._fieldList.push(this._courtCompanion);
}
CourtingActivity.prototype = Object.create(Activity.prototype);
CourtingActivity.prototype.processActivity = function(self,event) {
    var order, notes;
    var compChosen = self._courtCompanion.options[self._courtCompanion.selectedIndex].text;
    if (compChosen) {
        var herSl = Number(compChosen.match(/\d+/)[0]);
        var herName = compChosen.match(/^(.*) \(/)[1];
        var herNotes = compChosen.match(/^(?:.*) \((.*)\)/)[1];
        var cost = herSl*3;
        var diff = self._sl-herSl;
        var odds = 2;
        order = "Court "+herName;
        if (diff < 3)   { odds = 3; }
        if (diff < 0)   { odds = 4; }
        if (diff < -3)  { odds = 5; }
        if (diff ===-6) { odds = 6; }
        if (diff < -6) {
            notes = "Courting "+herName+" is impossible for one with your low social level";
        } else {
            notes = "Courting "+herName+" costs a minimum of "+cost+" Crowns. You need a "+odds+" or better on 1d6 to succeed. Modifiers: -1 if she already has a partner";
            if (herNotes.indexOf("Wealthy") === -1) {
                notes += ", +1 for each extra expenditure of "+cost+" Crowns";
            }
            notes += ". A roll of 1 always fails. ";
            var gain = 1;
            if (herNotes.indexOf("Stunning") !== -1) {
                gain += 1;
            }
            if (herSl > self._sl) {
                gain += (herSl - self._sl);
            }
            notes += " If successful you gain ";
            if (gain === 1) {
                notes += " a status point";
            } else {
                notes += ""+gain+" status points";
            }
            notes += " for being the companion of "+herName+" this month";
            if (herNotes.indexOf("Wealthy") !== -1) {
                if (herSl > self._sl) {
                    notes += " and she will make small gifts to you in future months";
                } else {
                    notes += " and "+herName+"'s wealth will be able to keep her in the manner to which she is accustomed";
                }
            }
            notes += ".";
        }
    }
    return {order: order, notes: notes};
};

function VisitMistressActivity(weekrow) {
    Activity.call(this,weekrow,"Visit female companion",undefined);
}
VisitMistressActivity.prototype = Object.create(Activity.prototype);
VisitMistressActivity.prototype.processActivity = function(self,event) {
    return {order: "Visit Female Companion", notes: "No additional expense."};
};

function PracticeFencingActivity(weekrow,regiment,rapier,sabre,cutlass) {
    Activity.call(this,weekrow,"Practice Fencing","practice-details");
    this._rw = (regiment ? regiment.weapon : null);
    this._rapier = rapier;
    this._sabre = sabre;
    this._cutlass = cutlass;
    this._practiceRapier = document.getElementById(this._weekrow._prefix+"-practice-rapier");
    this._practiceSabre = document.getElementById(this._weekrow._prefix+"-practice-sabre");
    this._practiceCutlass = document.getElementById(this._weekrow._prefix+"-practice-cutlass");
    this._fieldList.push(this._practiceRapier,this._practiceSabre,this._practiceCutlass);
}
PracticeFencingActivity.prototype = Object.create(Activity.prototype);
PracticeFencingActivity.prototype.processActivity = function(self,event) {
    var order, notes;
    if (self._practiceRapier.checked) {
        order = "Practice Rapier";
        if ( this._rw !== "Rapier" ) {
            notes = "Your Practice costs "+self._rapier+" Crowns.";
        } else {
            notes = "Your regiment will train you for free.";
        }
    }
    if (self._practiceSabre.checked) {
        order = "Practice Sabre";
        if ( this._rw !== "Sabre" ) {
            notes = "Your Practice costs "+self._sabre+" Crowns.";
        } else {
            notes = "Your regiment will train you for free.";
        }
    }
    if (self._practiceCutlass.checked) {
        order = "Practice Cutlass";
        if ( this._rw !== "Cutlass" ) {
            notes = "Your Practice costs "+self._cutlass+" Crowns.";
        } else {
            notes = "Your regiment will train you for free.";
        }
    }
    return {order: order, notes: notes};
};

function OnDutyActivity(weekrow) {
    Activity.call(this,weekrow,"On Duty",undefined);
}
OnDutyActivity.prototype = Object.create(Activity.prototype);
OnDutyActivity.prototype.processActivity = function(self,event) {
    return {order: "On Duty", notes: ""};
};

function OtherActivity(weekrow) {
    Activity.call(this,weekrow,"Other",undefined);
}
OtherActivity.prototype = Object.create(Activity.prototype);
OtherActivity.prototype.processActivity = function(self,event) {
    return {order: "Other", notes: ""};
};

function ExtraActivity(weekrow,div) {
    Activity.call(this,weekrow,div.dataset.activity,undefined);
    this._notes = div.innerHTML;
}
ExtraActivity.prototype = Object.create(Activity.prototype);
ExtraActivity.prototype.processActivity = function(self,event) {
    return {order: this._action, notes: this._notes};
};

function WeekRow(cid,weekno,sl,club,regiment,rapier,sabre,cutlass) {
    Row.call(this,cid);
    
    this._prefix = cid+"-week-"+weekno;

    // Set up the activity choice list
    this._possibleActivitiesList = [];
    this._possibleActivitiesList.push(new OtherActivity(this));
    
    var extra = document.getElementById(this._prefix+"-extra");
    if (extra && extra.dataset.activity && extra.dataset.activity !== "") {
        this._possibleActivitiesList.push(new ExtraActivity(this,extra));
    }
    
    this._possibleActivitiesList.push(new ClubActivity(this,cid,sl,club));
    this._possibleActivitiesList.push(new BawdyhouseActivity(this,sl));
    this._possibleActivitiesList.push(new PlayhouseActivity(this,sl));
    this._possibleActivitiesList.push(new CourtingActivity(this,sl));
    this._possibleActivitiesList.push(new VisitMistressActivity(this));
    this._possibleActivitiesList.push(new PracticeFencingActivity(this,regiment,rapier,sabre,cutlass));
    this._possibleActivitiesList.push(new OnDutyActivity(this));
    
    this._choice = document.getElementById(this._prefix);
    this._choiceLabel = document.getElementById(this._prefix+"-label");
    this._text = document.getElementById(this._prefix+"-notes");
    this._choice.length = 0;
    var self = this;
    this._possibleActivitiesList.forEach(function(activity) {
        var option = document.createElement("option");
        option.text = activity.name;
        self._choice.add(option);
        activity.deselectActivity(); // Hide all the suboptions from the activity now - we'll show the current one later if necessary
    });

    // And start with the first activity in the array
    this._choice.selectedIndex = 0;
    if (this._inParis.checked) {
        this.enable(this._choice,this._choiceLabel);
        this._currentActivity = null; // Which will force processChange to select the actual activity (and set this._currentActivity to the actual first activity)
    } else {
        this.disable(this._choice,this.choiceLabel);
    }
    
    // Build the list of elements which will control us
    var elements = [this._choice, this._text];
    this._possibleActivitiesList.forEach(function(activity) {
        elements = elements.concat(activity.fields.filter(function(item){
            return elements.indexOf(item) < 0;
        }));
    });
    // And then get controlled by them
    elements.forEach(function(element){
        self.controlledBy(element);
    });
    
    this.addInnerHTML(document.getElementById(cid+"-week-"+weekno+"-effect"));
    this.addOrderField(document.getElementById(cid+"-form-week"+weekno));
    this.addDetailsField(document.getElementById(cid+"-form-week"+weekno+"-notes"));
    this.addNotesField(document.getElementById(cid+"-form-week"+weekno+"-effect"));
    
    this.processChange(this,undefined);
}

WeekRow.prototype = Object.create(Row.prototype);

Object.defineProperty(WeekRow.prototype, "activity", { get: function() {
    return (this._activity ? this._activity.name : "");
}});

WeekRow.prototype.processChange = function(self,event) {
    var results;

    if (self._inParis.checked) {
        self.enable(self._choice,self._choiceLabel);
        var chosenActivity = self._possibleActivitiesList[self._choice.selectedIndex];
        if (self._currentActivity) {
            self._currentActivity.deselectActivity(); // The state changes mean that we may need to 
        }
        self._currentActivity = chosenActivity;       // deslect and select the current activity even if
        if (self._currentActivity) {
            self._currentActivity.selectActivity();   // the current activity itself doesn't change
            results = self._currentActivity.processActivity(self._currentActivity,event);
        }
    } else {
        self.disable(self._choice,self._choiceLabel);
        if (self._currentActivity) {
            self._currentActivity.deselectActivity();
            self._currentActivity = null;
        }
        results = {order: "", notes:"Out of Paris"};
    }
    
    self.notifyOrder(results.order);
    self.notifyDetails(self._text.value);
    self.notifyCommentary(results.notes);
};
